@@ -124,6 +124,16 @@ class ParseJsonFieldFilterTest < Test::Unit::TestCase
       assert_equal(nil, rec['m'])
       assert_equal(nil, rec['n'])
     end
+    test 'replace json field' do
+      json_string_val = '{"a":{"b":"c"},"d":["e","f"],"g":97,"h":{"i":"j"}}'
+      orig_a_value = 'orig a value'
+      rec = emit_with_tag('tag', {'message'=>json_string_val, 'a'=>orig_a_value},'
+        merge_json_log false
+        replace_json_log true
+        json_fields message
+      ')
+      assert_equal({'a'=>{'b'=>'c'}, 'd'=>['e', 'f'], 'g'=>97, 'h'=>{'i'=>'j'}}, rec['message'])
+    end
     test 'no fallback if parsing error in given field' do
       # test that - skip1 is skipped, skip2 is attempted to parse and fail
       # jsonfield is skipped - message is logged at debug level
@@ -137,5 +147,16 @@ class ParseJsonFieldFilterTest < Test::Unit::TestCase
       debugit(@driver, 'in test')
       assert_match /\[debug\]: parse_json_field could not parse field \[skip1\] as JSON: value \[\{"bogusvalue\}\]/, @logs[0]
     end
+    test 'fallback if given field is nil' do
+      # test that - skip1 is skipped because it has a nil key
+      json_string_val2 = '{"k":{"b":"c"},"l":["e","f"],"m":97,"n":{"i":"j"}}'
+      orig_a_value = 'orig a value'
+      rec = emit_with_tag('tag', {'skip1'=>nil, 'a'=>orig_a_value, 'jsonfield'=>json_string_val2},'
+        merge_json_log true
+        json_fields skip1,jsonfield
+      ')
+      assert_equal({'skip1'=>nil, 'a'=>orig_a_value, 'jsonfield'=>json_string_val2}, rec)
+      debugit(@driver, 'in test')
+    end
   end
 end
